@@ -28,7 +28,7 @@ local runDebugger = corsixth.require("run_debugger")
 -- Increment each time a savegame break would occur
 -- and add compatibility code in afterLoad functions
 
-local SAVEGAME_VERSION = 134
+local SAVEGAME_VERSION = 138
 
 class "App"
 
@@ -1249,11 +1249,12 @@ function App:checkInstallFolder()
   -- Do a few more checks to make sure that commonly corrupted files are OK.
   local corrupt = {}
 
+  -- Check for file corruption for local files.
+  -- No check is done if the game is loaded from an ISO
   local function check_corrupt(path, correct_size)
-    local real_path = self.fs:getFilePath(path)
     -- If the file exists but is smaller than usual it is probably corrupt
-    if real_path then
-      local real_size = lfs.attributes(real_path, "size")
+    if self.fs:fileExists(path) then
+      local real_size = self.fs:fileSize(path)
       if real_size + 1024 < correct_size or real_size - 1024 > correct_size then
         corrupt[#corrupt + 1] = path .. " (Size: " .. math.floor(real_size/1024) .. " kB / Correct: about " .. math.floor(correct_size/1024) .. " kB)"
       end
@@ -1318,6 +1319,7 @@ function App:readDataFile(dir, filename)
   if filename == nil then
     dir, filename = "Data", dir
   end
+
   local data = assert(self.fs:readContents(dir .. pathsep .. filename))
   if data:sub(1, 3) == "RNC" then
     data = assert(rnc.decompress(data))
@@ -1395,11 +1397,13 @@ end
 --! which save game version it is. This was added after the Beta 8
 --! release, which is why the checks prior to that version aren't made.
 --!param version An optional value if you want to find what game version
--- a specific savegame verion is from.
+-- a specific savegame version is from.
 function App:getVersion(version)
   local ver = version or self.savegame_version
-  if ver > 134 then
+  if ver > 138 then
     return "Trunk"
+  elseif ver > 134 then
+    return "v0.64"
   elseif ver > 127 then
     return "v0.63"
   elseif ver > 122 then
@@ -1495,7 +1499,7 @@ function App:exit()
   self.running = false
 end
 
---! Exits the game completly without saving the config i.e. Alt+F4 for Quit Application
+--! Exits the game completely without saving the config i.e. Alt+F4 for Quit Application
 function App:abandon()
   self.running = false
 end
@@ -1557,7 +1561,7 @@ function App:checkForUpdates()
 
   -- Default language to use for the changelog if no localised version is available
   local default_language = "en"
-  local update_url = 'http://www.corsixth.com/check-for-updates'
+  local update_url = 'www.corsixth.com/check-for-updates'
   local current_version = self:getVersion()
 
   -- Only URLs that match this list of trusted domains will be accepted.
@@ -1581,6 +1585,14 @@ function App:checkForUpdates()
   local http = require("socket.http")
   local url = require("socket.url")
 
+  -- Safely attempt to use luasec
+  local hassec, _ = pcall(require, "ssl.https")
+  if hassec then
+    update_url = "https://" .. update_url
+  else
+    update_url = "http://" .. update_url
+  end
+
   print("Checking for CorsixTH updates...")
   local update_body, status, _ = http.request(update_url)
 
