@@ -40,6 +40,8 @@ end
 if config_path ~= ourpath then
   config_path = pathconcat(config_path, "CorsixTH")
 end
+
+-- Config filename.
 config_name = "config.txt"
 
 -- Check for config.path.txt
@@ -78,6 +80,7 @@ if not check_dir_exists(config_path) then
   config_path = ourpath
 end
 
+-- Config file with full path as string.
 local config_filename = pathconcat(config_path, config_name)
 
 -- Create config.txt if it doesn't exist
@@ -134,14 +137,16 @@ local config_defaults = {
   shift_scroll_speed = 4,
   new_graphics_folder = nil,
   use_new_graphics = false,
-  check_for_updates = true
+  check_for_updates = true,
 }
+
 fi = io.open(config_filename, "r")
 local config_values = {}
 local needs_rewrite = false
 for key, value in pairs(config_defaults) do
   config_values[key] = value
 end
+
 if fi then
   -- Read all the values from the config file and put them in config_values. If at least one value is missing rewrite the configuration file.
   local file_contents = fi:read("*all")
@@ -165,6 +170,7 @@ if fi then
 else
   needs_rewrite = true
 end
+
 if needs_rewrite then
   fi = io.open(config_filename, "w")
   if fi then
@@ -218,7 +224,7 @@ if needs_rewrite then
 -- Note that audio will also be disabled if CorsixTH was compiled without the
 -- SDL_mixer library.
 -- ]=] .. '\n' ..
-'audio = '.. tostring(config_values.audio) .. '\n' .. [=[
+'audio = ' .. tostring(config_values.audio) .. '\n' .. [=[
 
 --------------------------------------------- CUSTOM GAME MENU ----------------------------------------------
 -- These settings can also be changed from the opening menu screen in the custom games menu
@@ -413,19 +419,16 @@ new_graphics_folder = nil -- [[X:\ThemeHospital\Graphics]]
 screenshots = nil -- [[X:\ThemeHospital\Screenshots]]
 
 -------------------------------------------------------------------------------------------------------------------------
--- High quality (MP3 rather than MIDI) audio replacements.
--- If you want to listen to high quality MP3 audio rather than the original XMI
--- (MIDI) audio, then follow these steps:
---  1) Find MP3 versions of the original tracks (for example the remixes by ZR
+-- If you want to listen to non-Theme-Hospital music, then follow these steps:
+--  1) Find updated versions of the original tracks (for example the remixes by ZR
 --     from http://www.a-base.dds.nl/temp/ThemeHospital_ZRRemix.zip ) or any
 --     other music you want to listen to.
---  2) Ensure that SMPEG.dll (or equivalent for your platform) is present.
---  3) Uncomment the next line and point it to where the mp3s are.
---  4) If you want to change the names of songs ingame, make a file called
+--  2) Uncomment the next line and point it to where the music files are.
+--  3) If you want to change the names of songs ingame, make a file called
 --     "names.txt" and write the file name on one row, followed by the desired
 --     ingame name on the next row.
 --
-audio_mp3 = nil -- [[X:\ThemeHospital\Music]]
+audio_music = nil -- [[X:\ThemeHospital\Music]]
 
  ----------------------------------------------- SPECIAL SETTINGS ----------------------------------------------
 -- These settings can only be changed here
@@ -433,7 +436,7 @@ audio_mp3 = nil -- [[X:\ThemeHospital\Music]]
 -- Audio playback settings.
 -- These can be commented out to use the default values from the game binary.
 -- Note: On some platforms, these settings may not effect MIDI playback - only
--- sound effects and MP3 audio. If you are experiencing poor audio playback,
+-- sound effects and music audio. If you are experiencing poor audio playback,
 -- then try doubling the buffer size.
 -- ]=] .. '\n' ..
 'audio_frequency = ' .. tostring(config_values.audio_frequency) .. '\n' ..
@@ -479,6 +482,9 @@ audio_mp3 = nil -- [[X:\ThemeHospital\Music]]
 'scroll_speed = ' .. tostring(config_values.scroll_speed) .. '\n' ..
 'shift_scroll_speed = ' .. tostring(config_values.shift_scroll_speed) .. '\n' .. [=[
 
+
+-------------------------------------------------------------------------------------------------------------------------
+
 ------------------------------------------------ CAMPAIGN MENU -----------------------------------------------
 -- By default your computer log in will be your name in the game.  You can change it in the
 -- campaign menu or between the brace brackets below [[YOUR NAME]].
@@ -496,4 +502,313 @@ audio_mp3 = nil -- [[X:\ThemeHospital\Music]]
   end
 end
 
-return config_filename, config_values
+-- Hotkey filename.
+local hotkeys_name = "hotkeys.txt"
+
+-- Hotkey file with full path as string.
+local hotkeys_filename = pathconcat(config_path, hotkeys_name)
+
+-- Defaults for hotkeys.
+local hotkeys_defaults = {
+  global_confirm = "return",
+  global_confirm_alt = "e",
+  global_cancel = "escape",
+  global_cancel_alt = "q",
+  global_fullscreen_toggle = {"alt", "return"},
+  global_exitApp = {"alt", "f4"},
+  global_resetApp = {"shift", "f10"},
+  global_releaseMouse = {"ctrl", "f10"},
+  global_connectDebugger = {"ctrl", "c"},
+  global_showLuaConsole = "f12",
+  global_runDebugScript = {"shift", "d"},
+  global_screenshot = {"ctrl", "s"},
+  global_stop_movie = "escape",
+  global_window_close = "escape",
+  global_stop_movie_alt = "q",
+  global_window_close_alt = "q",
+  ingame_showmenubar = "escape",
+  ingame_showCheatWindow = "f11",
+  ingame_pause = "p",
+  ingame_gamespeed_slowest = "1",
+  ingame_gamespeed_slower = "2",
+  ingame_gamespeed_normal = "3",
+  ingame_gamespeed_max = "4",
+  ingame_gamespeed_thensome = "5",
+  ingame_gamespeed_speedup = "z",
+  ingame_scroll_up = "up",
+  ingame_scroll_down = "down",
+  ingame_scroll_left = "left",
+  ingame_scroll_right = "right",
+  ingame_scroll_shift = "shift",
+  ingame_zoom_in = "=",
+  ingame_zoom_in_more = {"shift", "="},
+  ingame_zoom_out = "-",
+  ingame_zoom_out_more = {"shift", "-"},
+  ingame_setTransparent = "x",
+  ingame_toggleAdvisor = {"shift", "a"},
+  ingame_poopLog = {"ctrl", "d"},
+  ingame_poopStrings = {"ctrl", "t"},
+  ingame_toggleAnnouncements = {"alt", "a"},
+  ingame_toggleSounds = {"alt", "s"},
+  ingame_toggleMusic = {"alt", "m"},
+  ingame_panel_bankManager = "f1",
+  ingame_panel_bankStats = "f2",
+  ingame_panel_staffManage = "f3",
+  ingame_panel_townMap = "f4",
+  ingame_panel_casebook = "f5",
+  ingame_panel_research = "f6",
+  ingame_panel_status = "f7",
+  ingame_panel_charts = "f8",
+  ingame_panel_policy = "f9",
+  ingame_panel_map_alt = "t",
+  ingame_panel_research_alt = "r",
+  ingame_panel_casebook_alt = "c",
+  ingame_panel_casebook_alt02 = {"shift", "c"},
+  ingame_panel_buildRoom = "f",
+  ingame_panel_furnishCorridor = "g",
+  ingame_panel_editRoom = "v",
+  ingame_panel_hireStaff = "b",
+  ingame_loadMenu = {"shift", "l"},
+  ingame_saveMenu = {"shift", "s"},
+  ingame_restartLevel = {"shift", "r"},
+  ingame_quitLevel = {"shift", "q"},
+  ingame_quickSave = {"alt", "shift", "s"},
+  ingame_quickLoad = {"alt", "shift", "l"},
+  ingame_openFirstMessage = "m",
+  ingame_toggleInfo = "i",
+  ingame_jukebox = "j",
+  ingame_rotateobject = "space",
+  ingame_patient_gohome = "h",
+  ingame_storePosition_1 = {"alt", "1"},
+  ingame_storePosition_2 = {"alt", "2"},
+  ingame_storePosition_3 = {"alt", "3"},
+  ingame_storePosition_4 = {"alt", "4"},
+  ingame_storePosition_5 = {"alt", "5"},
+  ingame_storePosition_6 = {"alt", "6"},
+  ingame_storePosition_7 = {"alt", "7"},
+  ingame_storePosition_8 = {"alt", "8"},
+  ingame_storePosition_9 = {"alt", "9"},
+  ingame_storePosition_0 = {"alt", "0"},
+  ingame_recallPosition_1 = {"ctrl", "1"},
+  ingame_recallPosition_2 = {"ctrl", "2"},
+  ingame_recallPosition_3 = {"ctrl", "3"},
+  ingame_recallPosition_4 = {"ctrl", "4"},
+  ingame_recallPosition_5 = {"ctrl", "5"},
+  ingame_recallPosition_6 = {"ctrl", "6"},
+  ingame_recallPosition_7 = {"ctrl", "7"},
+  ingame_recallPosition_8 = {"ctrl", "8"},
+  ingame_recallPosition_9 = {"ctrl", "9"},
+  ingame_recallPosition_0 = {"ctrl", "0"},
+}
+
+-- Clear the loaded file variable.
+fi = io.open(hotkeys_filename, "r")
+local hotkeys_values = {}
+local hotkeys_needs_rewrite = false
+for key, value in pairs(hotkeys_defaults) do
+  hotkeys_values[key] = serialize(value)
+end
+
+-- If the file opened successfully...
+if fi then
+  local file_contents = fi:read("*all")
+  fi:close()
+
+  for key, _ in pairs(hotkeys_defaults) do
+    local ind = string.find(file_contents, "\n" .. "%s*" .. key .. "%s*=")
+
+    -- If we couldn't find the key in the hotkeys.txt file...
+    if not ind then
+      hotkeys_needs_rewrite = true
+    else
+      ind = ind + (string.find(file_contents, key, ind) - ind) + string.len(key)
+      ind = string.find(file_contents, "=", ind) + 1
+
+      ind = string.find(file_contents, "[%[{]", ind)
+      hotkeys_values[key] = string.sub(file_contents, ind, string.find(file_contents, "[\n]", ind + 1) - 1)
+    end
+  end
+else
+  hotkeys_needs_rewrite = true
+end
+
+--
+if hotkeys_needs_rewrite then
+  -- The config file that will be written is divided into separate strings, which are concatenated when they are written into the file.
+  -- This is done to avoid a "Too many C levels" error.
+  local string_01 = [=[
+--------------------------CorsixTH Hotkey Mappings File-------------------------
+-- Lines starting with two dashes (like this one) are ignored.
+-- Text settings should have their values between double square braces, e.g.
+-- Number settings should not have anything around their value, and complex
+-- settings should be surrounded by curly brackets.
+--
+-- EXAMPLES
+--   setting = [[value]]
+--   setting = 42
+--   setting = { 42, [[value]] }
+
+-----------------------------------Global Keys----------------------------------
+-- These are global keys to be used at anytime while the application is open.
+--]=] .. '\n' ..
+'global_confirm = ' .. hotkeys_values.global_confirm .. '\n' ..
+'global_confirm_alt = ' .. hotkeys_values.global_confirm_alt .. '\n' ..
+'global_cancel = ' .. hotkeys_values.global_cancel .. '\n' ..
+'global_cancel_alt = ' .. hotkeys_values.global_cancel_alt .. '\n' ..
+'global_fullscreen_toggle = ' .. hotkeys_values.global_fullscreen_toggle .. '\n' ..
+'global_exitApp = ' .. hotkeys_values.global_exitApp .. '\n' ..
+'global_resetApp = ' .. hotkeys_values.global_resetApp .. '\n' ..
+'global_releaseMouse = ' .. hotkeys_values.global_releaseMouse .. '\n' ..
+'global_connectDebugger = ' .. hotkeys_values.global_connectDebugger .. '\n' ..
+'global_showLuaConsole = ' .. hotkeys_values.global_showLuaConsole .. '\n' ..
+'global_runDebugScript = ' .. hotkeys_values.global_runDebugScript .. '\n' ..
+'global_screenshot = ' .. hotkeys_values.global_screenshot .. '\n' ..
+'global_stop_movie = ' .. hotkeys_values.global_stop_movie .. '\n' ..
+'global_window_close = ' .. hotkeys_values.global_window_close .. '\n' ..
+'global_stop_movie_alt =' .. hotkeys_values.global_stop_movie_alt .. '\n' ..
+'global_window_close_alt =' .. hotkeys_values.global_window_close_alt .. '\n' .. [=[
+
+-----------------------------------Scroll Keys----------------------------------
+-- These are the keys to be used to scroll the camera around in-game.
+-- ]=] .. '\n' ..
+'ingame_scroll_up = ' .. hotkeys_values.ingame_scroll_up .. '\n' ..
+'ingame_scroll_down = ' .. hotkeys_values.ingame_scroll_down .. '\n' ..
+'ingame_scroll_left = ' .. hotkeys_values.ingame_scroll_left .. '\n' ..
+'ingame_scroll_right = ' .. hotkeys_values.ingame_scroll_right .. '\n' ..
+'ingame_scroll_shift = ' .. hotkeys_values.ingame_scroll_shift .. '\n' .. [=[
+
+--------------------------------------Zoom--------------------------------------
+-- These are keys used to zoom the camera in and out.
+-- ]=] .. '\n' ..
+'ingame_zoom_in = '.. hotkeys_values.ingame_zoom_in .. '\n' ..
+'ingame_zoom_in_more = ' .. hotkeys_values.ingame_zoom_in_more .. '\n' ..
+'ingame_zoom_out = ' .. hotkeys_values.ingame_zoom_out .. '\n' ..
+'ingame_zoom_out_more = ' .. hotkeys_values.ingame_zoom_out_more .. '\n' .. [=[
+
+----------------------------------In-Game Menus---------------------------------
+-- These are quick keys to show the in-game menu bar and some other windows.
+-- ]=] .. '\n' ..
+'ingame_showmenubar = ' .. hotkeys_values.ingame_showmenubar .. '\n' ..
+'ingame_showCheatWindow = ' .. hotkeys_values.ingame_showCheatWindow .. '\n' ..
+'ingame_loadMenu = ' .. hotkeys_values.ingame_loadMenu .. '\n' ..
+'ingame_saveMenu = ' .. hotkeys_values.ingame_saveMenu .. '\n' ..
+'ingame_jukebox = ' .. hotkeys_values.ingame_jukebox .. '\n' ..
+'ingame_openFirstMessage = ' .. hotkeys_values.ingame_openFirstMessage .. '\n' .. [=[]=]
+
+  local string_02 = [=[
+
+----------------------------------Speed Control---------------------------------
+-- These pause and control the speed of the game.
+-- ]=] .. '\n' ..
+'ingame_pause = ' .. hotkeys_values.ingame_pause .. '\n' ..
+'ingame_gamespeed_slowest = ' .. hotkeys_values.ingame_gamespeed_slowest .. '\n' ..
+'ingame_gamespeed_slower = ' .. hotkeys_values.ingame_gamespeed_slower .. '\n' ..
+'ingame_gamespeed_normal = ' .. hotkeys_values.ingame_gamespeed_normal .. '\n' ..
+'ingame_gamespeed_max = ' .. hotkeys_values.ingame_gamespeed_max .. '\n' ..
+'ingame_gamespeed_thensome = ' .. hotkeys_values.ingame_gamespeed_thensome .. '\n' ..
+'ingame_gamespeed_speedup = ' .. hotkeys_values.ingame_gamespeed_speedup .. '\n' .. [=[
+
+------------------------------In-Game Bottom Panel------------------------------
+-- These open in-game panel windows like the town map or the "build room" dialog.
+-- ]=] .. '\n' ..
+'ingame_panel_bankManager = ' .. hotkeys_values.ingame_panel_bankManager .. '\n' ..
+'ingame_panel_bankStats = ' .. hotkeys_values.ingame_panel_bankStats .. '\n' ..
+'ingame_panel_staffManage = ' .. hotkeys_values.ingame_panel_staffManage .. '\n' ..
+'ingame_panel_townMap = ' .. hotkeys_values.ingame_panel_townMap .. '\n' ..
+'ingame_panel_casebook = ' .. hotkeys_values.ingame_panel_casebook .. '\n' ..
+'ingame_panel_research = ' .. hotkeys_values.ingame_panel_research .. '\n' ..
+'ingame_panel_status = ' .. hotkeys_values.ingame_panel_status .. '\n' ..
+'ingame_panel_charts = ' .. hotkeys_values.ingame_panel_charts .. '\n' ..
+'ingame_panel_policy = ' .. hotkeys_values.ingame_panel_policy .. '\n' ..
+'ingame_panel_map_alt = ' .. hotkeys_values.ingame_panel_map_alt .. '\n' ..
+'ingame_panel_research_alt = ' .. hotkeys_values.ingame_panel_research_alt .. '\n' ..
+'ingame_panel_casebook_alt = ' .. hotkeys_values.ingame_panel_casebook_alt .. '\n' ..
+'ingame_panel_casebook_alt02 = ' .. hotkeys_values.ingame_panel_casebook_alt02 .. '\n' ..
+'ingame_panel_buildRoom = ' .. hotkeys_values.ingame_panel_buildRoom .. '\n' ..
+'ingame_panel_furnishCorridor = ' .. hotkeys_values.ingame_panel_furnishCorridor .. '\n' ..
+'ingame_panel_editRoom = ' .. hotkeys_values.ingame_panel_editRoom .. '\n' ..
+'ingame_panel_hireStaff = ' .. hotkeys_values.ingame_panel_hireStaff .. '\n' .. [=[
+
+----------------------------------Rotate Object---------------------------------
+-- This key rotates objects while they are being placed.
+-- ]=] .. '\n' ..
+'ingame_rotateobject = ' .. hotkeys_values.ingame_rotateobject .. '\n' .. [=[
+
+-----------------------------------Quick Keys-----------------------------------
+-- These are keys for quick saving and loading, and for quickly restarting and quitting the level.
+-- ]=] .. '\n' ..
+'ingame_quickSave = ' .. hotkeys_values.ingame_quickSave .. '\n' ..
+'ingame_quickLoad = ' .. hotkeys_values.ingame_quickLoad .. '\n' ..
+'ingame_restartLevel = ' .. hotkeys_values.ingame_restartLevel .. '\n' ..
+'ingame_quitLevel = ' .. hotkeys_values.ingame_quitLevel .. '\n' .. [=[
+
+---------------------------------Set Transparent--------------------------------
+-- While held down any walls will be transparent, allowing you to see behind them.
+-- ]=] .. '\n' ..
+'ingame_setTransparent = ' .. hotkeys_values.ingame_setTransparent .. '\n' .. [=[
+]=]
+
+local string_03 = [=[
+
+----------------------------Store and Recall Position---------------------------
+-- These keys store and recall camera positions. If you press the key(s) that correspond to
+-- "ingame_recallPosition_1" while looking over the operating room, for instance, and then
+-- you move the camera away from there, you can press "ingame_recallPosition_1" whenever
+-- you want to go back to the operating room instantly.
+-- ]=] .. '\n' ..
+'ingame_storePosition_1 = ' .. hotkeys_values.ingame_storePosition_1 .. '\n' ..
+'ingame_storePosition_2 = ' .. hotkeys_values.ingame_storePosition_2 .. '\n' ..
+'ingame_storePosition_3 = ' .. hotkeys_values.ingame_storePosition_3 .. '\n' ..
+'ingame_storePosition_4 = ' .. hotkeys_values.ingame_storePosition_4 .. '\n' ..
+'ingame_storePosition_5 = ' .. hotkeys_values.ingame_storePosition_5 .. '\n' ..
+'ingame_storePosition_6 = ' .. hotkeys_values.ingame_storePosition_6 .. '\n' ..
+'ingame_storePosition_7 = ' .. hotkeys_values.ingame_storePosition_7 .. '\n' ..
+'ingame_storePosition_8 = ' .. hotkeys_values.ingame_storePosition_8 .. '\n' ..
+'ingame_storePosition_9 = ' .. hotkeys_values.ingame_storePosition_9 .. '\n' ..
+'ingame_storePosition_0 = ' .. hotkeys_values.ingame_storePosition_0 .. '\n' ..
+'ingame_recallPosition_1 = ' .. hotkeys_values.ingame_recallPosition_1 .. '\n' ..
+'ingame_recallPosition_2 = ' .. hotkeys_values.ingame_recallPosition_2 .. '\n' ..
+'ingame_recallPosition_3 = ' .. hotkeys_values.ingame_recallPosition_3 .. '\n' ..
+'ingame_recallPosition_4 = ' .. hotkeys_values.ingame_recallPosition_4 .. '\n' ..
+'ingame_recallPosition_5 = ' .. hotkeys_values.ingame_recallPosition_5 .. '\n' ..
+'ingame_recallPosition_6 = ' .. hotkeys_values.ingame_recallPosition_6 .. '\n' ..
+'ingame_recallPosition_7 = ' .. hotkeys_values.ingame_recallPosition_7 .. '\n' ..
+'ingame_recallPosition_8 = ' .. hotkeys_values.ingame_recallPosition_8 .. '\n' ..
+'ingame_recallPosition_9 = ' .. hotkeys_values.ingame_recallPosition_9 .. '\n' ..
+'ingame_recallPosition_0 = ' .. hotkeys_values.ingame_recallPosition_0 .. '\n' .. [=[
+
+---------------------------------Toggle Various---------------------------------
+-- These toggle various things. The names tell all.
+-- ]=] .. '\n' ..
+'ingame_toggleAnnouncements = ' .. hotkeys_values.ingame_toggleAnnouncements .. '\n' ..
+'ingame_toggleSounds = ' .. hotkeys_values.ingame_toggleSounds .. '\n' ..
+'ingame_toggleMusic = ' .. hotkeys_values.ingame_toggleMusic .. '\n' ..
+'ingame_toggleAdvisor = ' .. hotkeys_values.ingame_toggleAdvisor .. '\n' ..
+'ingame_toggleInfo = ' .. hotkeys_values.ingame_toggleInfo .. '\n' .. [=[
+
+------------------------------------Dump Log------------------------------------
+-- These keys dump logs. And strings, if too much fiber was taken.
+-- ]=] .. '\n' ..
+'ingame_poopLog = ' .. hotkeys_values.ingame_poopLog .. '\n' ..
+'ingame_poopStrings = ' .. hotkeys_values.ingame_poopStrings .. '\n' .. [=[
+
+--------------------------------Patient, Go Home--------------------------------
+-- This sends a patient home. Also a good anime episode name.
+-- ]=] .. '\n' ..
+'ingame_patient_gohome = ' .. hotkeys_values.ingame_patient_gohome .. '\n' .. [=[
+]=]
+  fi = io.open(hotkeys_filename, "w")
+  if fi then
+    fi:write(string_01 .. string_02 .. string_03)
+  fi:close()
+  end
+end
+
+for k, str_val in pairs(hotkeys_values) do
+  local status, lua_val = pcall(loadstring_envcall('return ' .. str_val), {})
+  if status then
+    hotkeys_values[k] = lua_val
+  end
+end
+
+return config_filename, config_values, hotkeys_filename, hotkeys_values, hotkeys_defaults
