@@ -18,7 +18,7 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE. --]]
 
-dofile "dialogs/place_objects"
+corsixth.require("dialogs.place_objects")
 
 class "UIEditRoom" (UIPlaceObjects)
 
@@ -30,8 +30,6 @@ function UIEditRoom:UIEditRoom(ui, room_type)
   -- NB: UIEditRoom:onCursorWorldPositionChange is called by the UIPlaceObjects
   -- constructor, hence the initialisation of required fields prior to the call.
   self.UIPlaceObjects(self, ui)
-  self:addKeyHandler("return", self.confirm) -- UIPlaceObjects does not need this
-  self:addKeyHandler("keypad enter", self.confirm)
 
   local app = ui.app
   local blue_red_swap = self.anims.Alt32_BlueRedSwap
@@ -88,6 +86,13 @@ function UIEditRoom:UIEditRoom(ui, room_type)
   self.mouse_down_y = false
   self.mouse_cell_x = 0
   self.mouse_cell_y = 0
+
+  self:registerKeyHandlers()
+end
+
+function UIEditRoom:registerKeyHandlers()
+  self:addKeyHandler("global_confirm", self.confirm) -- UIPlaceObjects does not need this
+  self:addKeyHandler("global_confirm_alt", self.confirm)
 end
 
 function UIEditRoom:close(...)
@@ -258,7 +263,7 @@ local function isHumanoidObscuringArea(humanoid, x1, x2, y1, y2)
       if (x1 == humanoid.tile_x or x2 == humanoid.tile_x) or
           (y1 == humanoid.tile_y or y2 == humanoid.tile_y) then
         -- Humanoid not in the rectangle, but might be walking into it
-        local action = humanoid.action_queue[1]
+        local action = humanoid:getCurrentAction()
         if action.name ~= "walk" then
           return false
         end
@@ -293,19 +298,20 @@ function UIEditRoom:clearArea()
         humanoids_to_watch[entity] = true
 
         -- Try to make the humanoid leave the area
+        local current_action = entity:getCurrentAction()
         local meander = entity.action_queue[2]
         if meander and meander.name == "meander" then
           -- Interrupt the idle or walk, which will cause a new meander target
           -- to be chosen, which will be outside the blueprint rectangle
           meander.can_idle = false
-          local on_interrupt = entity.action_queue[1].on_interrupt
+          local on_interrupt = current_action.on_interrupt
           if on_interrupt then
-            entity.action_queue[1].on_interrupt = nil
-            on_interrupt(entity.action_queue[1], entity)
+            current_action.on_interrupt = nil
+            on_interrupt(current_action, entity)
           end
-        elseif entity.action_queue[1].name == "seek_room" or (meander and meander.name == "seek_room") then
+        elseif current_action.name == "seek_room" or (meander and meander.name == "seek_room") then
           -- Make sure that the humanoid doesn't stand idle waiting within the blueprint
-          if entity.action_queue[1].name == "seek_room" then
+          if current_action.name == "seek_room" then
             entity:queueAction(MeanderAction():setCount(1):setMustHappen(true), 0)
           else
             meander.done_walk = false
@@ -356,7 +362,7 @@ function UIEditRoom:onTick()
         -- The person might be dying (this check should probably be moved into
         -- isHumanoidObscuringArea, but I don't want to change too much right
         -- before a release).
-        if humanoid.action_queue[1].name == "die" then
+        if humanoid:getCurrentAction().name == "die" then
           if not humanoid.hospital then
             self.humanoids_to_watch[humanoid] = nil
           end
@@ -610,13 +616,15 @@ function UIEditRoom:returnToDoorPhase()
           break
         end
         if obj.object_type.id == "litter" then -- Silently remove litter from the world.
-          obj.remove()
+          obj:remove()
           break
         end
+        local obj_state = obj:getState()
         self.world:destroyEntity(obj)
         if not obj.master then
           self:addObjects({{
             object = TheApp.objects[obj.object_type.id],
+            state = obj_state,
             qty = 1
           }})
         end
@@ -631,7 +639,7 @@ function UIEditRoom:returnToDoorPhase()
   -- backup list of objects
   self.objects_backup = {}
   for k, o in pairs(self.objects) do
-    self.objects_backup[k] = { object = o.object, qty = o.qty }
+    self.objects_backup[k] = { object = o.object, qty = o.qty, state = o.state }
   end
 
   UIPlaceObjects.removeAllObjects(self, true)
@@ -699,7 +707,7 @@ function UIEditRoom:screenToWall(x, y)
   -- NB: Doors and windows cannot be placed on corner tiles, hence walls of corner tiles
   -- are never returned, and the nearest non-corner wall is returned instead. If they
   -- could be placed on corner tiles, then you would have to consider the interaction of
-  -- wall shadows with windows and doors, amonst other things.
+  -- wall shadows with windows and doors, amongst other things.
   -- Swing doors are allowed to be adjacent everywhere except the top corner.
   local modifier = 0
   local swinging = false
@@ -1159,7 +1167,7 @@ end
 --! param wall (string) original wall orientation
 --! return x (int) updated x value
 --! return y (int) updated y value
---! return x_mod (int) offest value to apply to tile count to determine relative position
+--! return x_mod (int) offset value to apply to tile count to determine relative position
 --! return y_mod (int) offset value to apply to tile count to determine relatitve position
 --! return wall (string) wall orientation style (only 2 styles)
 local function doorWallOffsetCalculations(x, y, wall)
@@ -1528,11 +1536,6 @@ function UIEditRoom:placeObject()
 end
 
 function UIEditRoom:afterLoad(old, new)
-  if old < 101 then
-    self:removeKeyHandler("enter")
-    self:addKeyHandler("return", self.confirm)
-  end
-  if old < 104 then
-    self:addKeyHandler("keypad enter", self.confirm)
-  end
+  UIPlaceObjects.afterLoad(self, old, new)
+  self:registerKeyHandlers()
 end
