@@ -580,6 +580,14 @@ function Humanoid:setNextAction(action, high_priority)
       if removed.object and removed.object:isReservedFor(self) then
         removed.object:removeReservedUser(self)
       end
+      if removed.is_entering then
+        local dest_room = self.world:getRoom(removed.x, removed.y)
+        self:unexpectFromRoom(dest_room)
+        if dest_room and removed.reserve_on_resume and removed.reserve_on_resume:isReservedFor(self) then
+          removed.reserve_on_resume:removeReservedUser(self)
+          dest_room:tryAdvanceQueue()
+        end
+      end
     end
   end
 
@@ -675,9 +683,9 @@ end
 -- Helper function for the common case of instructing a `Humanoid` to walk to
 -- a position on the map. Equivalent to calling `setNextAction` with a walk
 -- action.
---!param tile_x (integer) The X-component of the Lua tile co-ordinates of the
+--!param tile_x (integer) The X-component of the Lua tile coordinates of the
 -- tile to walk to.
---!param tile_y (integer) The Y-component of the Lua tile co-ordinates of the
+--!param tile_y (integer) The Y-component of the Lua tile coordinates of the
 -- tile to walk to.
 --!param must_happen (boolean, nil) If true, then the walk action will not be
 -- interrupted.
@@ -764,8 +772,8 @@ function Humanoid:tickDay()
 
   -- If it is too hot or too cold, start to decrease happiness and
   -- show the corresponding icon. Otherwise we could get happier instead.
-  local min_comfort_temp = 0.22 -- 11 degrees Celcius.
-  local max_comfort_temp = 0.36 -- 18 degrees Celcius.
+  local min_comfort_temp = 0.22 -- 11 degrees Celsius.
+  local max_comfort_temp = 0.36 -- 18 degrees Celsius.
   local decrease_factor = 0.10
   local increase_happiness = 0.005
 
@@ -814,7 +822,7 @@ function Humanoid:unregisterRoomBuildCallback(callback)
   if self.build_callbacks[callback] then
     self.build_callbacks[callback] = nil
   else
-    self.world:gameLog("Warning: Trying to remove nonexistant room build callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
+    self.world:gameLog("Warning: Trying to remove nonexistent room build callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
   end
 end
 
@@ -848,7 +856,7 @@ function Humanoid:unregisterRoomRemoveCallback(callback)
     self.world:unregisterRoomRemoveCallback(callback)
     self.remove_callbacks[callback] = nil
   else
-    self.world:gameLog("Warning: Trying to remove nonexistant room remove callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
+    self.world:gameLog("Warning: Trying to remove nonexistent room remove callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
   end
 end
 
@@ -870,7 +878,7 @@ function Humanoid:unregisterStaffChangeCallback(callback)
   if self.staff_change_callbacks and self.staff_change_callbacks[callback] then
     self.staff_change_callbacks[callback] = nil
   else
-    self.world:gameLog("Warning: Trying to remove nonexistant staff callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
+    self.world:gameLog("Warning: Trying to remove nonexistent staff callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
   end
 end
 
@@ -947,7 +955,7 @@ function Humanoid:tostring()
         distance = "nil"
       end
       local standing = "false"
-      if action:isStanding() then
+      if action.isStanding and action:isStanding() then
         standing = "true"
       end
       action_string = action_string .. " - Bench distance: " .. distance .. " Standing: " .. standing
@@ -965,3 +973,19 @@ function Humanoid:tostring()
   result = result .. "]"
   return result
 end
+
+--! Unexpects humanoid from a room, if validly entering this room
+--!param dest_room (Room) The room the humanoid maybe expected at
+function Humanoid:unexpectFromRoom(dest_room)
+  -- Unexpect the patient from a possible destination room.
+  if dest_room and dest_room.door.queue then
+    -- 1st condition checks normal room routing
+    -- 2nd checks patients going to toilets, doctors and nurses
+    -- and redundantly checks patients routed between rooms
+    if self.next_room_to_visit == dest_room or
+        (dest_room ~= self.next_room_to_visit and (not self:getRoom() or class.is(self, Staff))) then
+      dest_room.door.queue:unexpect(self)
+      dest_room.door:updateDynamicInfo()
+    end
+  end
+ end
\ No newline at end of file
