@@ -18,6 +18,10 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE. --]]
 
+corsixth.require("announcer")
+
+local AnnouncementPriority = _G["AnnouncementPriority"]
+
 class "Hospital"
 
 ---@type Hospital
@@ -47,7 +51,6 @@ function Hospital:Hospital(world, avail_rooms, name)
   self.acc_research_cost = 0
   self.acc_overdraft = 0
   self.acc_heating = 0
-  self.discover_autopsy_risk = 10
 
   -- The sum of all material values (tiles, rooms, objects).
   -- Initial value: hospital tile count * tile value + 20000
@@ -370,7 +373,7 @@ function Hospital:cashLow()
   }
   if hosp.balance < 2000 and hosp.balance >= -500 then
     hosp.world.ui.adviser:say(cashlowmessage[math.random(1, #cashlowmessage)])
-  elseif hosp.balance < -2000 and hosp.world.month > 8 then
+  elseif hosp.balance < -2000 and hosp.world:date():monthOfYear() > 8 then
     -- ideally this should be linked to the lose criteria for balance
     hosp.world.ui.adviser:say(_A.warnings.bankruptcy_imminent)
   end
@@ -658,6 +661,11 @@ function Hospital:afterLoad(old, new)
     self.ratholes = {}
   end
 
+  if old < 131 then
+    self.autopsy_discovered = nil
+    self.discover_autopsy_risk = nil
+  end
+
   -- Update other objects in the hospital (added in version 106).
   if self.epidemic then self.epidemic.afterLoad(old, new) end
   for _, future_epidemic in ipairs(self.future_epidemics_pool) do
@@ -687,8 +695,8 @@ function Hospital:countSittingStanding()
   local numberSitting = 0
   local numberStanding = 0
   for _, patient in ipairs(self.patients) do
-    local pat_action = patient.action_queue[1]
-    if pat_action.name == "idle" then
+    local pat_action = patient:getCurrentAction()
+    if pat_action.name == "idle" and not patient:getRoom() then
       numberStanding = numberStanding + 1
     elseif pat_action.name == "use_object" and pat_action.object.object_type.id == "bench" then
       numberSitting = numberSitting + 1
@@ -700,105 +708,93 @@ end
 
 -- A range of checks to help a new player. These are set days apart and will show no more than once a month
 function Hospital:checkFacilities()
-  if self.hospital and self:isPlayerHospital() then
+  local current_date = self.world:date()
+  local day = current_date:dayOfMonth()
+  -- All messages are shown after first 4 months if respective conditions are met
+  if self:isPlayerHospital() and current_date >= Date(1,5) then
     -- If there is no staff room, remind player of the need to build one
-    if self.world.day == 3 and not self:hasRoomOfType("staff_room") then
-      if self.world.month > 4 and not self.staff_room_msg then
-        self:noStaffroom_msg()
-      elseif self.world.year > 1 then
-        self:noStaffroom_msg()
-      end
+    if not self.staff_room_msg and day == 3 and not self:hasRoomOfType("staff_room") then
+      self:noStaffroom_msg()
     end
     -- If there is no toilet, remind player of the need to build one
-    if self.world.day == 8 and not self:hasRoomOfType("toilets") then
-      if self.world.month > 4 and not self.toilet_msg then
-        self:noToilet_msg()
-      elseif self.world.year > 1 then
-        self:noToilet_msg()
-      end
+    if not self.toilet_msg and day == 8 and not self:hasRoomOfType("toilets") then
+      self:noToilet_msg()
     end
-    -- How are we for seating, if there are plenty then praise is due, if not the player is warned
-    -- We don't want to see praise messages about seating every month, so randomise the chances of it being shown
-    -- check the seating : standing ratio of waiting patients
-    -- find all the patients who are currently waiting around
-    local show_msg = math.random(1, 4)
-    local numberSitting, numberStanding = self:countSittingStanding()
 
-    -- If there are patients standing then maybe the seating is in the wrong place!
-    -- set to 5% (standing:seated) if there are more than 50 patients or 20% if there are less than 50.
-    -- If this happens for 10 days in any month you are warned about seating unless you have already been warned that month
-    -- So there are now two checks about having enough seating, if either are called then you won't receive praise. (may need balancing)
-    if self.patientcount < 50 then
-      if numberStanding > math.min(numberSitting / 5) then
+    if not self.bench_msg then
+      -- How are we for seating, if there are plenty then praise is due, if not the player is warned
+      -- check the seating : standing ratio of waiting patients
+      -- find all the patients who are currently waiting around
+      local numberSitting, numberStanding = self:countSittingStanding()
+
+      -- If there are patients standing then maybe the seating is in the wrong place!
+      -- set to 5% (standing:seated) if there are more than 50 patients or 20% if there are less than 50.
+      -- If this happens for 10 days in any month you are warned about seating unless you have already been warned that month
+      -- So there are now two checks about having enough seating, if either are called then you won't receive praise. (may need balancing)
+      local number_standing_threshold = numberSitting / (self.patientcount < 50 and 5 or 20)
+      if numberStanding > number_standing_threshold then
         self.seating_warning = self.seating_warning + 1
+        if self.seating_warning >= 10 then
+          self:warningBench()
+        end
       end
-    else
-      if numberStanding > math.min(numberSitting / 20) then
-        self.seating_warning = self.seating_warning + 1
-      end
-      if self.seating_warning >= 10 and not self.bench_msg then
-        self:warningBench()
-        self.seating_warning = 0
-      end
-    end
-    if self.world.day == 12 and show_msg  == 4 and not self.bench_msg and
-        (self.world.year > 1 or (self.world.year == 1 and self.world.month > 4)) then
-      -- If there are less patients standing than sitting (1:20) and there are more benches than patients in the hospital
-      -- you have plenty of seating.  If you have not been warned of standing patients in the last month, you could be praised.
-      if self.world.object_counts.bench > self.patientcount then
-        self:praiseBench()
-      -- Are there enough benches for the volume of patients in your hospital?
-      elseif self.world.object_counts.bench < self.patientcount then
-        self:warningBench()
+
+      if day == 12 then
+        -- If there are less patients standing than sitting (1:20) and there are more benches than patients in the hospital
+        -- you have plenty of seating.  If you have not been warned of standing patients in the last month, you could be praised.
+
+        -- We don't want to see praise messages about seating every month, so randomise the chances of it being shown
+        local show_praise = math.random(1, 4) == 4
+        if self.world.object_counts.bench > self.patientcount and show_praise then
+          self:praiseBench()
+        -- Are there enough benches for the volume of patients in your hospital?
+        elseif self.world.object_counts.bench < self.patientcount then
+          self:warningBench()
+        end
       end
     end
 
-    -- Make players more aware of the need for radiators and how hot or cold the patients and staff are
-    -- If there are no radiators remind the player from May onwards
-    if self.world.object_counts.radiator == 0 and self.world.month > 4 and self.world.day == 15 then
+    -- Make players more aware of the need for radiators
+    if self.world.object_counts.radiator == 0 then
       self.world.ui.adviser:say(_A.information.initial_general_advice.place_radiators)
     end
-    -- Now to check how warm or cold patients and staff are.  So that we are not bombarded with warmth
+
+    -- Now to check how warm or cold patients and staff are. So that we are not bombarded with warmth
     -- messages if we are told about patients then we won't be told about staff as well in the same month
     -- And unlike TH we don't want to be told that anyone is too hot or cold when the boiler is broken do we!
-    if not self.heating_broke then
-      if not self.warmth_msg and self.world.day == 15 then
+    if not self.warmth_msg and not self.heating_broke then
+      if day == 15 then
         local warmth = self:getAveragePatientAttribute("warmth", 0.3) -- Default value does not result in a message.
-        if (self.world.year > 1 or self.world.month > 4) and warmth < 0.22 then
+        if warmth < 0.22 then
           self:warningTooCold()
-        elseif self.world.month > 4 and warmth >= 0.36 then
+        elseif warmth >= 0.36 then
           self:warningTooHot()
         end
       end
       -- Are the staff warm enough?
-      if not self.warmth_msg and self.world.day == 20 then
+      if day == 20 then
         local avgWarmth = self:getAverageStaffAttribute("warmth", 0.25) -- Default value does not result in a message.
-        if (self.world.year > 1 or self.world.month > 4) and avgWarmth < 0.22 then
+        if avgWarmth < 0.22 then
           self.world.ui.adviser:say(_A.warnings.staff_very_cold)
-        elseif self.world.month > 4 and avgWarmth >= 0.36 then
+        elseif avgWarmth >= 0.36 then
           self.world.ui.adviser:say(_A.warnings.staff_too_hot)
         end
       end
     end
+
     -- Are the patients in need of a drink
-    if not self.thirst_msg and self.world.day == 24 then
+    if not self.thirst_msg and day == 24 then
       local thirst = self:getAveragePatientAttribute("thirst", 0) -- Default value does not result in a message.
-      if self.world.year == 1 and self.world.month > 4 then
-        if thirst > 0.8 then
-          self.world.ui.adviser:say(_A.warnings.patients_very_thirsty)
-        elseif thirst > 0.6 then
-          self:warningThirst()
-        end
-      elseif self.world.year > 1 then
-        if thirst > 0.9 then
-          self.world.ui.adviser:say(_A.warnings.patients_very_thirsty)
-        elseif thirst > 0.6 then
-          self:warningThirst()
-        end
+      local thirst_threshold = current_date:year() == 1 and 0.8 or 0.9
+      if thirst > thirst_threshold then
+        self.world.ui.adviser:say(_A.warnings.patients_very_thirsty)
+      elseif thirst > 0.6 then
+        self:warningThirst()
       end
     end
+
     -- reset all the messages on 28th of each month
-    if self.world.day == 28 then
+    if day == 28 then
       self.staff_room_msg = false
       self.toilet_msg = false
       self.bench_msg = false
@@ -811,7 +807,7 @@ function Hospital:checkFacilities()
 end
 
 --! Called each tick, also called 'hours'. Check hours_per_day in
---! world.lua to see how many times per day this is.
+--! date.lua to see how many times per day this is.
 function Hospital:tick()
 -- add some random background sounds, ringing phones, coughing, belching etc
   self:countPatients()
@@ -881,8 +877,8 @@ function Hospital:getHeliportSpawnPosition()
 end
 
 --[[ Test if a given map tile is part of this hospital.
-!param x (integer) The 1-based X co-ordinate of the tile to test.
-!param y (integer) The 1-based Y co-ordinate of the tile to test.
+!param x (integer) The 1-based X coordinate of the tile to test.
+!param y (integer) The 1-based Y coordinate of the tile to test.
 ]]
 function Hospital:isInHospital(x, y)
   local flags = self.world.map.th:getCellFlags(x, y)
@@ -893,7 +889,7 @@ function Hospital:coldWarning()
     "sorry002.wav", "sorry004.wav",
   }
   if announcements and self:isPlayerHospital() then
-    self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)])
+    self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.Normal)
   end
 end
 function Hospital:hotWarning()
@@ -901,7 +897,7 @@ function Hospital:hotWarning()
     "sorry003.wav", "sorry004.wav",
   }
   if announcements and self:isPlayerHospital() then
-    self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)])
+    self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.Normal)
   end
 end
 -- Called when the hospitals's boiler has broken down.
@@ -1030,7 +1026,7 @@ function Hospital:onEndDay()
   if breakdown == 1 and not self.heating_broke and self.boiler_can_break and
       self.world.object_counts.radiator > 0 then
     if tonumber(self.world.map.level_number) then
-      if self.world.map.level_number == 1 and (self.world.month > 5 or self.world.year > 1) then
+      if self.world.map.level_number == 1 and (self.world:date() >= Date(1,6)) then
         self:boilerBreakdown()
       elseif self.world.map.level_number > 1 then
         self:boilerBreakdown()
@@ -1041,22 +1037,8 @@ function Hospital:onEndDay()
   end
 
   -- Calculate heating cost daily.  Divide the monthly cost by the number of days in that month
-  local month_length = {
-    31, -- Jan
-    28, -- Feb
-    31, -- Mar
-    30, -- Apr
-    31, -- May
-    30, -- Jun
-    31, -- Jul
-    31, -- Aug
-    30, -- Sep
-    31, -- Oct
-    30, -- Nov
-    31, -- Dec
-  }
   local radiators = self.world.object_counts.radiator
-  local heating_costs = (((self.radiator_heat * 10) * radiators) * 7.50) / month_length[self.world.month]
+  local heating_costs = (((self.radiator_heat * 10) * radiators) * 7.50) / self.world:date():lastDayOfMonth()
   self.acc_heating = self.acc_heating + heating_costs
 
   if self:isPlayerHospital() then dailyUpdateRatholes(self) end
@@ -1066,6 +1048,7 @@ end
 function Hospital:onEndMonth()
   -- Spend wages
   local wages = 0
+  local current_month = self.world:date():monthOfYear()
   for _, staff in ipairs(self.staff) do
     wages = wages + staff.profile.wage
   end
@@ -1130,10 +1113,10 @@ function Hospital:onEndMonth()
   end
 
   -- Check for equipment getting available
-  self.research:checkAutomaticDiscovery(self.world.month + 12 * (self.world.year - 1))
+  self.research:checkAutomaticDiscovery(self.world:date():monthOfGame())
 
   -- Add some interesting statistics.
-  self.statistics[self.world.month + 1 + 12 * (self.world.year - 1)] = {
+  self.statistics[self.world:date():monthOfGame() + 1] = {
     money_in = self.money_in,
     money_out = self.money_out,
     wages = wages,
@@ -1147,18 +1130,18 @@ function Hospital:onEndMonth()
   self.money_out = 0
 
   -- make players aware of the need for a receptionist and desk.
-  if (self:isPlayerHospital() and not self:hasStaffedDesk()) and self.world.year == 1 then
-    if self.receptionist_count ~= 0 and self.world.month > 2 and not self.receptionist_msg then
+  if (self:isPlayerHospital() and not self:hasStaffedDesk()) and self.world:date():year() == 1 then
+    if self.receptionist_count ~= 0 and current_month > 2 and not self.receptionist_msg then
       self.world.ui.adviser:say(_A.warnings.no_desk_6)
       self.receptionist_msg = true
-    elseif self.receptionist_count == 0 and self.world.month > 2 and self.world.object_counts["reception_desk"] ~= 0  then
+    elseif self.receptionist_count == 0 and current_month > 2 and self.world.object_counts["reception_desk"] ~= 0  then
       self.world.ui.adviser:say(_A.warnings.no_desk_7)
     --  self.receptionist_msg = true
-    elseif self.world.month == 3 then
+    elseif current_month == 3 then
       self.world.ui.adviser:say(_A.warnings.no_desk, true)
-    elseif self.world.month == 8 then
+    elseif current_month == 8 then
       self.world.ui.adviser:say(_A.warnings.no_desk_1, true)
-    elseif self.world.month == 11 then
+    elseif current_month == 11 then
       if self.visitors == 0 then
         self.world.ui.adviser:say(_A.warnings.no_desk_2, true)
       else
@@ -1207,7 +1190,7 @@ function Hospital:onEndYear()
   -- this will replicate that. I have still to check other levels above 5 to
   -- see if there are other large increases.
   -- TODO Hall of fame and shame
-  if self.world.year == 3 and self.world.map.level_number == 3 then
+  if self.world:date():year() == 3 and self.world.map.level_number == 3 then
     -- adds the extra to salary in level 3 year 3
     self.player_salary = self.player_salary + math.random(8000,20000)
   end
@@ -1247,9 +1230,11 @@ function Hospital:createEmergency(emergency)
     if self:hasRoomOfType(emergency.disease.treatment_rooms[no_rooms]) then
       room_name = nil
     end
-    local added_info = _S.fax.emergency.cure_possible
-    -- TODO: Differentiate if a drug is needed, add drug effectiveness. Add undiscovered treatment.
-    -- added_info = _S.fax.emergency.cure_not_possible
+
+    local casebook = self.disease_casebook[random_disease.id]
+    local added_info = casebook.drug and
+        _S.fax.emergency.cure_possible_drug_name_efficiency:format(emergency.disease.name, casebook.cure_effectiveness)
+        or _S.fax.emergency.cure_possible
     if room_name then
       if staff_available then
         added_info = _S.fax.emergency.cure_not_possible_build:format(room_name) .. "."
@@ -1276,7 +1261,7 @@ function Hospital:createEmergency(emergency)
         {text = _S.fax.emergency.choices.refuse, choice = "refuse_emergency"},
       },
     }
-    self.world.ui.bottom_panel:queueMessage("emergency", message, nil, 24*20, 2) -- automatically refuse after 20 days
+    self.world.ui.bottom_panel:queueMessage("emergency", message, nil, Date.hoursPerDay() * 16, 2) -- automatically refuse after 16 days
     created_one = true
   end
   return created_one
@@ -1287,7 +1272,7 @@ function Hospital:resolveEmergency()
   local emer = self.emergency
   local rescued_patients = emer.cured_emergency_patients
   for _, patient in ipairs(self.emergency_patients) do
-    if patient and not patient.cured and not patient:getRoom() then
+    if patient and not patient.cured and not patient.dead and not patient:getRoom() then
       patient:die()
     end
   end
@@ -1444,7 +1429,7 @@ function Hospital:determineIfContagious(patient)
   -- The patient isn't contagious if these conditions aren't passed
   local reduce_months = level_config.ReduceContMonths or 14
   local reduce_people = level_config.ReduceContPeepCount or 20
-  local date_in_months = self.world.month + (self.world.year - 1)*12
+  local date_in_months = self.world:date():monthOfGame()
 
   if potentially_contagious and date_in_months > reduce_months and
       self.num_visitors > reduce_people then
@@ -1644,8 +1629,8 @@ at least one of the following integer fields: `spend`, `receive`.
 ]]
 function Hospital:logTransaction(transaction)
   transaction.balance = self.balance
-  transaction.day = self.world.day
-  transaction.month = self.world.month
+  transaction.day = self.world:date():dayOfMonth()
+  transaction.month = self.world:date():monthOfYear()
   while #self.transactions > 20 do
     self.transactions[#self.transactions] = nil
   end
@@ -1693,7 +1678,7 @@ function Hospital:initStaff()
         added_staff = false
       end
       if added_staff then
-        local staff = self.world:newEntity("Staff", 2)
+        local staff = self.world:newEntity(profile.humanoid_class, 2)
         staff:setProfile(profile)
         -- TODO: Make a somewhat "nicer" placing algorithm.
         staff:setTile(self.world.map.th:getCameraTile(1))
@@ -1710,6 +1695,7 @@ function Hospital:addStaff(staff)
   self.staff[#self.staff + 1] = staff
   -- Cost of hiring staff:
   self:spendMoney(staff.profile.wage, _S.transactions.hire_staff .. ": " .. staff.profile.name)
+  self:notifyOfStaffChange(staff)
 end
 
 function Hospital:addPatient(patient)
@@ -1785,6 +1771,8 @@ end
 --!param staff (Staff) Staff member to remove.
 function Hospital:removeStaff(staff)
   RemoveByValue(self.staff, staff)
+  -- update all messages for waiting patients
+  self:notifyOfStaffChange(staff)
 end
 
 --! Remove a patient from the hospital.
@@ -1836,7 +1824,7 @@ function Hospital:changeReputation(reason, disease, valueChange)
   if self.reputation_above_threshold then self:checkReputation() end
 end
 
---! Check whether the reputation is still above the treshold.
+--! Check whether the reputation is still above the threshold.
 function Hospital:checkReputation()
   local level_config = self.world.map.level_config
   if level_config.awards_trophies then
@@ -1937,8 +1925,9 @@ function Hospital:getAveragePatientAttribute(attribute, default_value)
   local sum = 0
   local count = 0
   for _, patient in ipairs(self.patients) do
+    local tx, ty = patient.tile_x, patient.tile_y
     -- Some patients (i.e. Alien) may not have the attribute in question, so check for that
-    if patient.attributes[attribute] then
+    if tx and ty and self:isInHospital(tx, ty) and patient.attributes[attribute] then
       sum = sum + patient.attributes[attribute]
       count = count + 1
     end
@@ -2229,13 +2218,13 @@ function Hospital:searchForHandymanTask(handyman, taskType)
     if canContinue then
       if v.assignedHandyman then
         if v.assignedHandyman.fired then
-          v.assignedHandyman = nil
+          v.assignedHandyman:unassignTask()
         elseif not v.assignedHandyman.hospital then
           -- This should normally never be the case. If the handyman doesn't belong to a hsopital
           -- then they should not have any tasks assigned to them however it was previously possible
           -- We need to tidy up to make sure the task can be reassigned.
           print("Warning: Orphaned handyman is still assigned a task. Removing.")
-          v.assignedHandyman = nil
+          v.assignedHandyman:unassignTask()
         else
           local assignedDistance = self.world:getPathDistance(v.tile_x, v.tile_y, v.assignedHandyman.tile_x, v.assignedHandyman.tile_y)
           if assignedDistance ~= false then
@@ -2380,3 +2369,11 @@ function Hospital:computePriceLevelImpact(patient, casebook)
     self.world.ui.adviser:say(_A.warnings.fair_prices:format(casebook.disease.name))
   end
 end
+
+--! Notify patients of a change to hospital staff members
+--!param staff (Staff) Changed staff member subject of notification
+function Hospital:notifyOfStaffChange(staff)
+  for _, patient in pairs(self.patients) do
+    patient:notifyOfStaffChange(staff)
+  end
+end
