@@ -22,11 +22,14 @@ local TH = require"TH"
 local ipairs, _G, table_remove
     = ipairs, _G, table.remove
 
-corsixth.require("entities.patient")
-corsixth.require("entities.staff")
-corsixth.require("entities.vip")
-corsixth.require("entities.grim_reaper")
-corsixth.require("entities.inspector")
+corsixth.require("entities.humanoids.patient")
+corsixth.require("entities.humanoids.staff.doctor")
+corsixth.require("entities.humanoids.staff.nurse")
+corsixth.require("entities.humanoids.staff.handyman")
+corsixth.require("entities.humanoids.staff.receptionist")
+corsixth.require("entities.humanoids.vip")
+corsixth.require("entities.humanoids.grim_reaper")
+corsixth.require("entities.humanoids.inspector")
 corsixth.require("staff_profile")
 corsixth.require("hospital")
 corsixth.require("epidemic")
@@ -34,6 +37,9 @@ corsixth.require("calls_dispatcher")
 corsixth.require("research_department")
 corsixth.require("entity_map")
 corsixth.require("date")
+corsixth.require("announcer")
+
+local AnnouncementPriority = _G["AnnouncementPriority"]
 
 --! Manages entities, rooms, and the date.
 class "World"
@@ -57,6 +63,7 @@ local earthquake_warning_period = 600 -- hours between warning and real thing
 local earthquake_warning_length = 25 -- length of early warning quake
 
 function World:World(app)
+  self.app = app
   self.map = app.map
   self.wall_types = app.walls
   self.object_types = app.objects
@@ -205,19 +212,18 @@ end
 --! Register key shortcuts for controlling the world (game speed, etc.)
 function World:setUI(ui)
   self.ui = ui
-  self.ui:addKeyHandler("P", self, self.pauseOrUnpause, "Pause")
-  self.ui:addKeyHandler("1", self, self.setSpeed, "Slowest")
-  self.ui:addKeyHandler("2", self, self.setSpeed, "Slower")
-  self.ui:addKeyHandler("3", self, self.setSpeed, "Normal")
-  self.ui:addKeyHandler("4", self, self.setSpeed, "Max speed")
-  self.ui:addKeyHandler("5", self, self.setSpeed, "And then some more")
 
-  self.ui:addKeyHandler("=", self, self.adjustZoom,  1)
-  self.ui:addKeyHandler({"shift", "="}, self, self.adjustZoom, 5)
-  self.ui:addKeyHandler("+", self, self.adjustZoom,  1)
-  self.ui:addKeyHandler({"shift", "+"}, self, self.adjustZoom, 5)
-  self.ui:addKeyHandler("-", self, self.adjustZoom, -1)
-  self.ui:addKeyHandler({"shift", "-"}, self, self.adjustZoom, -5)
+  self.ui:addKeyHandler("ingame_pause", self, self.pauseOrUnpause, "Pause")
+  self.ui:addKeyHandler("ingame_gamespeed_slowest", self, self.setSpeed, "Slowest")
+  self.ui:addKeyHandler("ingame_gamespeed_slower", self, self.setSpeed, "Slower")
+  self.ui:addKeyHandler("ingame_gamespeed_normal", self, self.setSpeed, "Normal")
+  self.ui:addKeyHandler("ingame_gamespeed_max", self, self.setSpeed, "Max speed")
+  self.ui:addKeyHandler("ingame_gamespeed_thensome", self, self.setSpeed, "And then some more")
+
+  self.ui:addKeyHandler("ingame_zoom_in", self, self.adjustZoom,  1)
+  self.ui:addKeyHandler("ingame_zoom_in_more", self, self.adjustZoom, 5)
+  self.ui:addKeyHandler("ingame_zoom_out", self, self.adjustZoom, -1)
+  self.ui:addKeyHandler("ingame_zoom_out_more", self, self.adjustZoom, -5)
 end
 
 function World:adjustZoom(delta)
@@ -578,7 +584,7 @@ function World:tickEarthquake()
     -- start of warning quake
     if self.next_earthquake.warning_timer == earthquake_warning_period then
       self.ui:beginShakeScreen(0.2)
-      self.ui:playAnnouncement(announcements[math.random(1, #announcements)])
+      self.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.Critical)
     end
 
     -- end of warning quake
@@ -596,7 +602,7 @@ function World:tickEarthquake()
 
       -- start of real earthquake
       if self.next_earthquake.warning_timer <= 0 then
-        self.ui:playAnnouncement(announcements[math.random(1, #announcements)])
+        self.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.Critical)
       end
     end
 
@@ -806,7 +812,7 @@ function World:notifyRoomRemoved(room)
   end
 end
 
---! Clear all internal caches which are dependant upon map state / object position
+--! Clear all internal caches which are dependent upon map state / object position
 function World:clearCaches()
   self.idle_cache = {}
 end
@@ -1334,6 +1340,21 @@ function World:nextEarthquake()
   end
 end
 
+-- Earthquake override from cheat menu
+function World:createEarthquake()
+  --make sure an earthquake isn't already happening
+  if not self.next_earthquake.active then
+    self.next_earthquake.start_day = self.game_date:dayOfMonth()
+    self.next_earthquake.start_month = self.game_date:monthOfGame()
+    if self.next_earthquake.size == nil then
+      --forcefully make an earthquake if none left in level file
+      self.next_earthquake.size = math.random(1,6) -- above 6 seems disastrous
+      self.next_earthquake.remaining_damage = self.next_earthquake.size
+      self.next_earthquake.damage_timer = earthquake_damage_time
+      self.next_earthquake.warning_timer = earthquake_warning_period
+    end
+  end
+end
 
 --! Checks if all goals have been achieved or if the player has lost.
 --! Returns a table that always contains a state string ("win", "lose" or "nothing").
@@ -2252,7 +2273,7 @@ function World:getRoomNameAndRequiredStaffName(room_id)
   local room_name, required_staff, staff_name
   for _, room in ipairs(TheApp.rooms) do
     if room.id == room_id then
-      room_name = room.name
+      room_name = room.long_name
       required_staff = room.required_staff
     end
   end
@@ -2329,6 +2350,9 @@ function World:afterLoad(old, new)
   if new - 20 > self.original_savegame_version then
     self.ui:addWindow(UIInformation(self.ui, {_S.information.very_old_save}))
   end
+
+  self:setUI(self.ui)
+
   -- insert global compatibility code here
   if old < 4 then
     self.room_built = {}
@@ -2406,10 +2430,6 @@ function World:afterLoad(old, new)
     self.animation_manager = TheApp.animation_manager
     self.anim_length_cache = nil
   end
-  if old < 16 then
-    self.ui:addKeyHandler("+", self, self.adjustZoom,  1)
-    self.ui:addKeyHandler("-", self, self.adjustZoom, -1)
-  end
   if old < 17 then
     -- Added another object
     local pathsep = package.config:sub(1, 1)
@@ -2553,10 +2573,6 @@ function World:afterLoad(old, new)
       end
     end
   end
-  if old < 77 then
-    self.ui:addKeyHandler({"shift", "+"}, self, self.adjustZoom,  5)
-    self.ui:addKeyHandler({"shift", "-"}, self, self.adjustZoom, -5)
-  end
 
   if old < 103 then
     -- If a room has patients who no longer exist in its
